#!/usr/local/bin/perl
# $Header: /afs/andrew.cmu.edu/system/src/host/pkgtools/001/RCS/cmp_env.pl,v 1.2 1994/01/11 22:40:54 ww0r Exp $

@c1 = ();
@c2 = ();
$name = '';
$ver = '';
$volname = '';
undef %vol1;
undef %vol2;

select(STDOUT); $|=1;  

($env1, $env2) = @ARGV;
print "Fetching collections for $env1...";
@c1 = &getusedcollections($env1);
print "done.\n";

print "Fetching collections for $env2...";
@c2 = &getusedcollections($env2);
print "done.\n";

print "Doing AFS volume name lookups for $env1...";
foreach (@c1) {
  $volname = &getvolumename("$env1/$_");
  ($name,$ver) = (split(/\./,$volname))[1,2];
  if (defined($ver)) {
    $vol1{$name} = $ver;
  } else {
    $vol1{$name} = 'no version';
  }
}
print "done.\n";

print "Doing AFS volume name lookups for $env2...";
foreach (@c2) {
  $volname = &getvolumename("$env2/$_");
  ($name,$ver) = (split(/\./,$volname))[1,2];
  if (defined($ver)) {
    $vol2{$name} = $ver;
  } else {
    $vol2{$name} = 'no version';
  }
}

print "done.\n\n\n";

&print_diff();

exit(0);

sub
getusedcollections {
  local($path) = @_;
  local(*DP);
  local(@coll);
  local(%dirs);
  local($name, $ver);
  local(@verlist);

  opendir(DP, $path) || die "Invalid path for collections: $path: $!\n";
  @coll =  sort grep(-d "$path/$_",
		     grep(!/^\.\.?$/,readdir(DP)));
  closedir(DP);

  # load up all the versions
  foreach (@coll) {
    ($name,$ver) = split(/\./);
    $ver = 'no version'
      if (!defined $ver);
    if ($dirs{$name})  {
      $dirs{$name} .= "/$ver";
    } else {
      $dirs{$name} = $ver;
    }
  }

  # go through and add only the highest versions.
  # check the lowest values for negatives and ditch those
  @coll = ();
  foreach (sort keys %dirs) {
    @verlist = reverse sort split(/\//,$dirs{$_});
    if ($verlist[0] eq "no version"){
      push(@coll,"$_");
    } elsif ($verlist[$#verlist] > 0) {
      push(@coll,"$_.$verlist[0]");
    } 
  }
  
  @coll;
}
  
sub
getvolumename {
  local($path) = @_;
  local(*FD);
  local($str);

  open(FD, "fs lsm $path|") || die "error running fs on path $path: $!\n";
  $str = <FD>;
  $str = (split(/\#/,$str))[1];
  chop($str);chop($str);	# remove \n and \'
  $str;
}
		
sub
print_diff {
  local(%mark);
  local(@diff);
  local(@e1, @e2);
  local($nodiff) = 1;


  @e1 = sort keys %vol1;
  @e2 = sort keys %vol2;

  grep($mark{$_}++, @e2);
  @diff = grep(! $mark{$_}, @e1);
  print "Collections only in $env1: @diff\n"
  if (@diff);


  print "\nVolumes with different versions\n";
  print "-" x length("Volumes with different versions"), "\n";
  foreach (@e1) {
    if (($vol2{$_}) && ($vol2{$_} != $vol1{$_})) {
      $name = $_;
      $ver1 = $vol1{$_};
      $ver2 = $vol2{$_};
      write;
      $nodiff = 0;
    }
  }
  print "No differences found\n"
    if ($nodiff);
}

format STDOUT = 
@<<<<<<<<<<   @<<<<<    @<<<<<
$name         $ver1     $ver2
.

