#!/usr/local/bin/perl

$debug = 0;
$doing_devices = 0;
require 'ctime.pl';

while ($ARGV[0] =~ /^-/) {
  if ($ARGV[0] eq "-d") {
    $debug = 1;
  } elsif ($ARGV[0] eq "-s") {
    shift @ARGV;
    $sys = $ARGV[0];
  } else {
    &usage();
  }
  shift @ARGV;
}

&usage()
  if (@ARGV != 1);

($os_tree, $start_dir) = $ARGV[0] =~ /(.*os)(.*$)/;

if (! defined $sys) {
  $sys = `sys` ;
  chop($sys);
}

if (defined $os_tree) { 
  if ($os_tree =~ /\/nos$/) {
    $os_tree = "$os_tree/$start_dir";
    $start_dir = '';
  }
} else {
  $os_tree = "";
  $start_dir = $ARGV[0];
}

chop($start_dir)
    if ($start_dir =~ /.*\/$/);

if ($debug == 1) {
  open(UNDOLOG, ">/usr/tmp/undolog-modes.${sys}");
} else {
  open(UNDOLOG, ">/afs/.andrew.cmu.edu/wsadmin/support/pckg/undolog-modes.${sys}");
}

select(UNDOLOG);
$| = 1;
print UNDOLOG "#!/bin/sh\n";
select(STDOUT);
$| = 1;

&traverse($start_dir, 0);

exit(0);

sub 
printline {
  local($file, $owner, $group, $mode) = @_;
  if ((($mode & 0400) == 0) || (($mode & 0200) == 0)) {
    $newmode = $mode | 0600;
    if ($debug == 1) {
      printf "Would run chmod %3.3lo ${os_tree}${file}\n", $newmode;
    } else {
      chmod $newmode, "${os_tree}${file}";
    }
    print UNDOLOG "chmod ";
    printf UNDOLOG "%3.3lo", $mode;
    print UNDOLOG " ${os_tree}${file}\n";
  }
}


#
sub 
traverse {
  local($dir, $absolute)=@_;
  local($d);
  local(%inoar) = ();
  local(@dircache) = ();
  local(@filecache) = ();
  local(@linkcache) = ();
  local(@devcache) = ();
  local(*DIR);
  local($file, $rfile);
  
  $rdir= $os_tree . $dir;

  opendir(DIR, $rdir) || die "Can't open directory $rdir: $!\n";
  local(@dir) = grep (!/^\.\.?$/,readdir(DIR));
  closedir(DIR);
  
  foreach $d (@dir) {
    $file = "$dir/$d";
    $rfile = "$os_tree$file";

    ($ino, $mode, $nlink,$uid, $gid, $rdev) = (lstat($rfile))[1,2,3,4,5,6];

    $names{$uid} = $usermap{$uid} if (defined $usermap{$uid} && !defined $names{$uid});
    ($names{$uid}) = getpwuid($uid) if ! defined $names{$uid};
    $oname = $names{$uid};
    $groups{$gid} = $groupmap{$gid} if (defined $groupmap{$gid} && !defined $groups{$gid});
    ($groups{$gid}) = getgrgid($gid) if ! defined $groups{$gid};
    $gname = $groups{$gid};
    
    $mode = $mode & 0007777;
    
    # just in case, handle unknown users/groups
    $oname = "root" if (! defined($oname));
    
    #or common translations
    $gname = "wheel" if (! defined $gname || $gname eq "staff");
    
    # make hard links into symbolic links 
    # (package can not do hard links - sigh)
    if (defined $inoar{$ino}) {
      $inoar{$ino} = join('\\', $inoar{$ino},
			  join($;, $file, $oname, $gname, $mode));
      next;
    }

    # cache directory entries do we do not get huge %inoar wastage
    if (-d _) {
      push(@dircache, join($;, $file, $oname, 
			   $gname, $mode));
      next;
    }

    # has hard links to it
    if ($nlink > 1) {
      print STDERR "WARNING: Hard link detected for file $file.\n";
      print STDERR "         You should run soften.pl before running pckg.\n";
      $inoar{$ino} = $file;
    }
    
    # normal files
    if (-f _) {
      push(@filecache, join($;, $file, $oname, $gname, $mode));
      next;
    } 
  }				# end of for loop for files

  # emit hard link translations
  for (sort values %inoar) {
    ($prime, @sub) = split(/\\/);
    for (sort @sub) {
      ($file, $oname, $gname, $mode) = split(/$;/);
      push(@linkcache, join($;, $file, 
			    $prime, $oname, $gname, $mode));
    }
  }

  %inoar = ();

# --- output the lines
  # symlinks 

  # files 
  foreach (sort @filecache) {
    local($ftype);
    ($file, $oname, $gname, $mode) = split(/$;/);

    &printline($file, $oname, $gname, $mode);
  }

  #  directories
  for (sort @dircache) {
    ($file, $oname, $gname, $mode) = split(/$;/);
    &traverse($file, 1);
  }

  # clear directory cache
  @dircache = ();
}

sub 
usage {
  print "usage: fix-modes [-d] directory\n";
  print "\t-d\tEnable debugging mode.\n";
  exit 1;
}
