#!/usr/local/bin/perl
# $Id: pckg.pl,v 2.3 1995/12/07 21:22:57 wcw Exp wcw $
# pckg - generate a package description of a filesystem tree
#
# requires Perl 4.x (possibly 4.10 or greater)
#
# usage:
#	pckg [-d] [-f config_file] [-s sys_type] [directory]
#
# local andrew.cmu.edu caveats:
#
#	We have the convention of an OS tree, which is an image
#	of the vendor supplied OS from / on down. It resides on
#	AFS, and via package we can selectively link or copy
#	files across. This script is designed to create a complete
#	package description for an OS tree, which can then be edited
#	down for local purposes.
#
#	Here, we mount OS trees as
#
#	/afs/andrew.cmu.edu/@sys/os
#
#	pckg will attempt to discover where the root of the OS tree that you
#	point it at is. If you do not use .../os, you will need to modify how 
#	it pulls apart $ARGV[0] to get it right for your purposes. If it does
#	not think you've pointed it at an OS tree, it will just generate. This
#	is useful for creating package descriptions for /dev ala
#
#	pckg /dev
#
#	We also have the convention of the mpp macro ${machine} which will
#	resolve to the start of the appropriate OS tree. Again, you may
#	want to change it for local use. Another convention is that we
#	tend to map most ownerships down to root/wheel. If pckg cannot
#	figure out who owns the file (in the case that some vendor shipped
#	things owned by some bizzare userid), it will render the user down
#	to root and groups down to wheel. This may not suit all, but is
#	easy to change in the code.
#
#	Obplug: please look around /afs/andrew.cmu.edu/wsadmin/lib to see how
#	we do things after six years of dealing with package. It may provide
#	some ideas. wsadmin/public/src/public.proto is used by mpp (ignore
#	the public.@sys files)
#
# general notes:
#	- since AFS and package have problems with hard links, one should
#       run 'soften.pl' to break the hard links before trying to 
#       generate the package image files

$debug = 0;
$doing_devices = 0;
$config_file = '/afs/andrew.cmu.edu/wsadmin/support/pckg/pckg.generic';
require 'ctime.pl';

while ($ARGV[0] =~ /^-/) {
  if ($ARGV[0] eq "-d") {
    $debug = 1;
  } elsif ($ARGV[0] eq "-f") {
    shift @ARGV;
    $config_file = $ARGV[0];
  } elsif ($ARGV[0] eq "-s") {
    shift @ARGV;
    $sys = $ARGV[0];
  } elsif ($ARGV[0] eq "-l") {
    $light = 1;
  } else {
    &usage();
  }
  shift @ARGV;
}

&usage()
  if (@ARGV != 1);

($os_tree, $start_dir) = $ARGV[0] =~ /(.*os)(.*$)/;

if (! defined $sys) {
  $sys = `sys` ;
  chop($sys);
}

if (defined $os_tree) { 
  if ($os_tree =~ /\/nos$/) {
    $os_tree = "$os_tree/$start_dir";
    $start_dir = '';
  }
} else {
  $os_tree = "";
  $start_dir = $ARGV[0];
}

chop($start_dir)
    if ($start_dir =~ /.*\/$/);

&init();

$light = 1 if $lightlist{$start_dir};

print '### BEGIN ${wsadmin}/lib/sys/tree.' . "$sys" . "\n";
print "### This file is automatically generated by pckg.pl" . ' $Revision: 2.3 $ ' . "on ", &ctime(time);
print "### DO NOT EDIT\n";
&traverse($start_dir, $light, 0);
print '### END ${wsadmin}/lib/sys/tree.' . "$sys" . "\n";

exit(0);

sub 
printline {
  local($type, $file, $rfile, $owner, $group, $mode, $wrap) = @_;

  print "%ifndef has$file\n"
    if ($wrap);    
  print "$type $file $rfile $owner $group \${mode";
  printf "%3.3lo}\n" ,$mode;
  if ($wrap) {
    print "%define has$file\n";
    print "%endif\n"
  }
}


#
sub 
traverse {
  local($dir, $light, $absolute)=@_;
  local($d);
  local(%inoar) = ();
  local(@dircache) = ();
  local(@filecache) = ();
  local(@linkcache) = ();
  local(@devcache) = ();
  local(*DIR);
  local($file, $rfile);
  
  $rdir= $os_tree . $dir;

  opendir(DIR, $rdir) || die "Can't open directory $rdir: $!\n";
  local(@dir) = grep (!/^\.\.?$/,readdir(DIR));
  closedir(DIR);
  
  foreach $d (@dir) {
    $file = "$dir/$d";
    $rfile = "$os_tree$file";

    ($ino, $mode, $nlink,$uid, $gid, $rdev) = (lstat($rfile))[1,2,3,4,5,6];

    $names{$uid} = $usermap{$uid} if (defined $usermap{$uid} && !defined $names{$uid});
    ($names{$uid}) = getpwuid($uid) if ! defined $names{$uid};
    $oname = $names{$uid};
    $groups{$gid} = $groupmap{$gid} if (defined $groupmap{$gid} && !defined $groups{$gid});
    ($groups{$gid}) = getgrgid($gid) if ! defined $groups{$gid};
    $gname = $groups{$gid};
    
    $mode = $mode & 0007777;
    
    # just in case, handle unknown users/groups
    $oname = "root" if (! defined($oname));
    
    #or common translations
    $gname = "wheel" if (! defined $gname || $gname eq "staff");
    
    # symlinks
    if (-l _) {
      $link = readlink($rfile);
      
      local(@linkar) = split(/\//, $link);
      
      # don't do anything with an absolute link
      if ($linkar[0] ne "") {
	$up = 0;
	# rip off "." and ".." counting ".."s in $up
	while (1) {
	  # link to ./something - rip off ./
	  if ($linkar[0] eq ".") {
	    shift(@linkar);
	    next;
	  }
	  if ($linkar[0] eq "..") {
	    shift(@linkar);
	    $up++;
	    next;
	  }
	  #drop out when we have no .|..
	  last;
	}
	
	local(@dirar) = split(/\//, $dir);
	while ($up--) {
	  #rip from bottom this time
	  pop(@dirar);
	}
	
	$link = join('/', @dirar, @linkar);
	if ($#dirar == -1) {
	  #link went above /. odd, but legal
	  $link = "/$link";
	}
      }
      push(@linkcache, join($;,$file,$link,$oname,$gname,$mode));
      next;
    }
    
    # make hard links into symbolic links 
    # (package can not do hard links - sigh)
    if (defined $inoar{$ino}) {
      $inoar{$ino} = join('\\', $inoar{$ino},
			  join($;, $file, $oname, $gname, $mode));
      next;
    }

    # cache directory entries do we do not get huge %inoar wastage
    if (-d _) {
      push(@dircache, join($;, $file, $oname, 
			   $gname, $mode));
      next;
    }

    # has hard links to it
    if ($nlink > 1) {
      print STDERR "WARNING: Hard link detected for file $file.\n";
      print STDERR "         You should run soften.pl before running pckg.\n";
      $inoar{$ino} = $file;
    }
    
    # normal files
    if (-f _) {
      push(@filecache, join($;, $file, $oname, $gname, $mode));
      next;
    } 
    # device entry
    if (-b _ || -c _) {
      # some systems use longwords - detect
      if ($sys eq "hp700_ux90") {
	$major_dev = (($rdev >> 24)& 0x000000ff); # 8 bit major
	$minor_dev = $rdev & 0x00ffffff;    # 24 bit minors
      } else {
	if ($rdev & 0xffff0000) {
	  $major_dev = ($rdev >> 16) & 0x0000ffff;
	  $minor_dev = ($rdev) & 0x0000ffff;
	} else {
	  $major_dev = ($rdev >> 8) & 0x000000ff;
	  $minor_dev = ($rdev) & 0x000000ff;
	}
      }
      if (-b _) {
	$type = "B";
      } else {
	$type = "C";
      }
      push(@devcache, join($;,$type, $file, 
			    $major_dev, $minor_dev, 
			    $oname, $gname, $mode));
      next;
    }
  }				# end of for loop for files

  # emit hard link translations
  for (sort values %inoar) {
    ($prime, @sub) = split(/\\/);
    for (sort @sub) {
      ($file, $oname, $gname, $mode) = split(/$;/);
      push(@linkcache, join($;, $file, 
			    $prime, $oname, $gname, $mode));
    }
  }

  %inoar = ();

# --- output the lines
  # symlinks 
  foreach (sort @linkcache) {
    ($file, $link, $oname, $gname, $mode) = split(/$;/);
    &printline("LA", $file, $link, $oname, $gname, $mode, 1);
  }

  # files 
  foreach (sort @filecache) {
    local($ftype);
    ($file, $oname, $gname, $mode) = split(/$;/);

    # yuck... doing this manually...
    if ($rebootlist{$file}) {
      $ftype = 'FOQ';
    } else {
      $ftype = 'F';
    }

    if ($securelist{$file}) {
      print "%ifndef has$file\n";
      print "%ifdef secure\n";
      &printline($ftype, $file, '${machine}',$oname, $gname, $mode, 0);
      print "%else\n";
      ($oname,$gname,$mode) = split(/,/,$securelist{$file});
      &printline($ftype, $file, '${machine}',$oname, $gname, oct($mode), 0);
      print "%endif\n";
      print "%define has$file\n";
      print "%endif\n";
    } else {
      if ($permlist{$file}) {
        ($oname,$gname,$mode) = split(/,/,$permlist{$file});
        &printline($ftype, $file, '${machine}',$oname, $gname, oct($mode), 1);
      } else {
        &printline($ftype, $file, '${machine}',$oname, $gname, $mode,1);
      }
    }
  }

  # devices
  foreach (sort @devcache) {
    ($type, $file, $major_dev, $minor_dev, $oname, $gname, $mode) = split(/$;/);
    &printline($type, $file, "$major_dev $minor_dev", $oname, $gname, $mode, 1);
  }
  
  #  directories
  for (sort @dircache) {
    ($file, $oname, $gname, $mode) = split(/$;/);

    # light hierarchy
    if ($light) {
        if ($prunelist{$file}) {
		&printline("DR", $file, '${machine}', $oname, $gname, $mode, 0);
        } else {
		&printline("DR", $file, ' ', $oname, $gname, $mode, 0);
		&traverse($file, 1);
	}
	next;
    } 

    print "%ifndef has$file\n";
    print "%ifndef nocp$file\n";
    &printline('${cp' . $file . '}', $file, '${machine}', $oname, $gname, $mode,0);
    print "%else\n";
    if ($prunelist{$file}) {
	&printline("DR", $file, '${machine}', $oname, $gname, $mode,0);
    } else {
	&printline("DR", $file, '', $oname, $gname, $mode,0);
	&traverse($file, $light);
    }
    print "%endif\n";
    print "%define has$file\n";
    print "%endif $file\n";
  }
  
  # clear directory cache
  @dircache = ();
}


sub
init {

# don't buffer stdout
  select(STDOUT);
  $| = 1;
  
  system("mpp -Dsys=$sys -D$sys $config_file > /tmp/pckg.$$");
  die "error running mpp: rc = $?\n"
    if ($? != 0);

  &parse("/tmp/pckg.$$");
  unlink("/tmp/pckg.$$")
   unless ($debug); 
}

sub 
usage {
  print "usage: pckg [-d] [-f config_file] directory\n";
  print "\t-d\tEnable debugging mode.\n";
  print "\t-f\tSpecify an alternate configuration file.\n";
  print "\t-s\tSpecify an alternate system type.\n";
  exit 1;
}

sub parse {
    local($file) = @_;
    local(%valid) = ("rebootlist", 1, "prunelist", 1, "securelist", 1, "lightlist", 1, "permlist", 1, "usermap", 1, "groupmap", 1);
    local($curhash);
    local(*FD);
    local($line) = (0);

    open(FD, "<$file") || die "Error opening file: $file: $!\n";

    undef $curhash;

    while (<FD>) {
	$line++;
	s/\#.*$//;
	next if /^\s*$/;
	s/^\s*$//;

	if (/^BEGIN\s+(\S+)/) {
	    die "Error: $1 is an unknown list name (line $line)\n" if (! $valid{$1});
	    $curhash = $1;
	    next;
	}
	elsif (/^END/) {
	    undef $curhash;
	    next;
	}
	
	@ar = $_ =~ /(\S+)\s*(.*)$/;

	die "Error: unbound value $ar[0] (line $line)\n" if !defined $curhash;

	if (length($ar[1])) {
	    eval '$' . $curhash . "{'$ar[0]'} = '$ar[1]'";
	} else {
	    # no explicit value
	    eval '$' . $curhash . "{'$ar[0]'} = 1";
	}
    }

    die "Error: config not terminated by END\n" if defined $curhash;
}

# ***********************************************************
#         Copyright 1991,1994,1995 by Carnegie Mellon University
# 
#                       All Rights Reserved
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of CMU not be
# used in advertising or publicity pertaining to distribution of the
# software without specific, written prior permission.
# 
# CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
# CMU BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
# SOFTWARE.
# ******************************************************************/
# 
# end of pckg.pl
