#!/usr/local/bin/perl -w
# $Id: setup-sys.pl,v 1.1 1996/06/20 18:19:58 shadow Exp $
# 
# This perl script reads the configuration file specified on the command line
# and then goes through and (optionally creates), mounts, sets quota and access
# to the specified OS volumes.

$config_file = "/afs/andrew.cmu.edu/wsadmin/support/pckg/setup-depot.conf";

%names = '';
$doit = 1;
$verbose = 0;
$sys = '';
$betanameprefix = '';
$destnameprefix = '';
$deltanameprefix = '';
$prefix = '';
$objprefix = '';
$a = '';
$b = '';

while ($ARGV[0] =~ /^-/) {
  if ($ARGV[0] eq "-n")	 {
    $doit = 0;
  } elsif ($ARGV[0] eq "-v") {
    $verbose = 1;
  } elsif ($ARGV[0] eq "-r") {
    shift @ARGV;
    chdir($ARGV[0]) || die "Unable to chdir to $ARGV[0]: $!\n";
    print "Using $ARGV[0] as root directory\n"
      if ($verbose);
  }
  shift @ARGV;
}

if ($#ARGV != 1) {
  print STDERR 
  "usage: setup-depot [-n] [-v] [-r <dir> ] sys short_sys\n",
  "e.g. setup-depot -r /afs/.andrew.cmu.edu sun4_55 sun455\n",
  '$Revision: 1.1 $',
  "\n",
  "\t-n\tDon't really run the commands.\n",
  "\t-v\tBe verbose.\n",
  "\t-r <dir>\tUse <dir> as root directory.\n";

  exit(-1);
}

$sys = $ARGV[0];
$prefix = $ARGV[1];

if ($verbose) {
  print "Reading configuration file: $config_file\n";
  print "Using system $sys short sys $prefix\n";
}

&load_conf($config_file);

exit(0);

sub 
load_conf {
  local(*FD);
  local($cfile) = @_;
  local($set_prefix, $set_postfix, $set_beta, $set_sys, $set_delta, $set_dest, $set_files, $set_names, $set_dirs, $set_objprefix, $set_links);

  open(FD, "$cfile") || 
    die "Can't open configuration file: $cfile: $!\n";
  $set_prefix = 1;
  $set_postfix = 1;
  $set_beta = 1;
  $set_delta = 1;
  $set_dest = 1;
  $set_sys = 1;
  $set_objprefix = 1;

# skip leading foo
  while (<FD>) {
    next if /^#/;
    next if /^\s+$/;

    if (/^\s*names:\s*$/) {
      $set_sys = 0;
      $set_beta = 0;
      $set_postfix = 0;
      $set_objprefix = 0;
      $set_prefix = 0;
      $set_files = 0;
      $set_names = 1;
      $set_dirs = 0;
      $set_links = 0;
      next;
    }
    if (/^\s*links:\s*$/) {
      $set_sys = 0;
      $set_beta = 0;
      $set_postfix = 0;
      $set_objprefix = 0;
      $set_prefix = 0;
      $set_files = 0;
      $set_names = 0;
      $set_links = 1;
      $set_dirs = 0;
      next;
    }
    if (/^\s*dirs:\s*$/) {
      $set_sys = 0;
      $set_beta = 0;
      $set_postfix = 0;
      $set_objprefix = 0;
      $set_prefix = 0;
      $set_files = 0;
      $set_names = 0;
      $set_dirs = 1;
      $set_links = 0;
      next;
    }
    if (/^\s*files:\s*$/) {
      $set_sys = 0;
      $set_beta = 0;
      $set_postfix = 0;
      $set_objprefix = 0;
      $set_prefix = 0;
      $set_files = 1;
      $set_names = 0;
      $set_dirs = 0;
      $set_links = 0;
      next;
    }
    if ($set_names) {
      local($ename, $vname, $remainder);
      ($ename, $vname, $remainder) = split(/\s+/, $_, 3);
      chop $remainder;
      $names{$ename} = $vname;
    }
    if ($set_links) {
      local($dir);
      $dir = eval ("\"$_\"");
      &run("ln -s ${dir}");
    }
    if ($set_dirs) {
      local($dir, $foo);
      $foo = $_;
      foreach (sort bylength keys %names) {
        $ename = $_;
        $vname = $names{$ename};
        $dir = eval ("\"$foo\"");
        &run("mkdir ${dir}");
      }
    }
    if ($set_files) {
      local($path, $volname, $pathr, $volnamer);
      ($path, $volname) = split(/\s+/);
      foreach (sort bylength keys %names) {
        $ename = $_;
        $vname = $names{$ename};
	$pathr = eval ("\"$path\"");
	$volnamer =  eval ("\"$volname\"");
        &run("cp ${volnamer} ${pathr}");
      }
    }

    if ($set_beta && /^\s*betanameprefix/) {
      $betanameprefix = (split(/\s+/))[1];
      $set_beta = 0;
    }
    if ($set_dest && /^\s*destnameprefix/) {
      $destnameprefix = (split(/\s+/))[1];
      $set_dest = 0;
    }
    if ($set_delta && /^\s*deltanameprefix/) {
      $deltanameprefix = (split(/\s+/))[1];
      $set_delta = 0;
    }
    if ($set_prefix && /^\s*prefix/) {
      $prefix = (split(/\s+/))[1];
      $set_prefix = 0;
    }
    if ($set_objprefix && /^\s*objprefix/) {
      $objprefix = (split(/\s+/))[1];
      $set_objprefix = 0;
    }
    if ($set_sys && /^\s*sys/) {
      $sys = (split(/\s+/))[1];
      $set_sys = 0;
    }
  }
  close(FD);
}

sub
bylength {
  split(/\//,$a) <=> split(/\//,$b);
}

sub
run {
  local($cmd) = @_;

  if ($verbose) {
    print "running $cmd\n";
  }
  if ($doit) {
    system($cmd);
    if ($? ne 0) {
      print "Error running: $cmd: rc = $?\n";
      exit(-1);
    }
  }
}

# end of file
