#!/usr/local/bin/perl -w
# $Id: setup-sys.pl,v 1.2 1997/11/26 21:37:06 shadow Exp ad88 $
# 
# This perl script reads the configuration file specified on the command line
# and then goes through and (optionally creates), mounts, sets quota and access
# to the specified OS volumes.

$config_file = "/afs/andrew.cmu.edu/wsadmin/support/pckg/setup-sys.conf";

%made = '';
%mounted = '';
%names = '';
%acls = '';
$setacl = '';
$doit = 1;
$verbose = 0;
$create_vols = 0;
$server = '';
$part = '';
$sys = '';
$betanameprefix = '';
$destnameprefix = '';
$deltanameprefix = '';
$prefix = '';
$objprefix = '';
$postfix = '';
$a = '';
$b = '';

while ($ARGV[0] =~ /^-/) {
  if ($ARGV[0] eq "-n")	 {
    $doit = 0;
  } elsif ($ARGV[0] eq "-v") {
    $verbose = 1;
  } elsif ($ARGV[0] eq "-c") {
    $create_vols = 1;
    shift @ARGV;
    $server = shift @ARGV;
    $part = @ARGV[0];		# don't shift. will do it at the end of the loop
  } elsif ($ARGV[0] eq "-r") {
    shift @ARGV;
    chdir($ARGV[0]) || die "Unable to chdir to $ARGV[0]: $!\n";
    print "Using $ARGV[0] as root directory\n"
      if ($verbose);
  }
  shift @ARGV;
}

if ($#ARGV != 2) {
  print STDERR 
  "usage: setup-sys [-n] [-v] [-c server part] [-r <dir> ] sys short_sys vers\n",
  "e.g. setup-sys -c vice15 a -r /afs/.andrew.cmu.edu sun4_55 sun455 001\n",
  '$Revision: 1.2 $',
  "\n",
  "\t-n\tDon't really run the commands.\n",
  "\t-v\tBe verbose.\n",
  "\t-c\tCreate the volumes on server/part.\n",
  "\t-r <dir>\tUse <dir> as root directory.\n";

  exit(-1);
}

$sys = $ARGV[0];
$prefix = $ARGV[1];
$postfix = $ARGV[2];

if ($verbose) {
  print "Reading configuration file: $config_file\n";
  print "Creating system $sys with volume prefix $prefix, version $postfix\n";
  print "Create volumes = $create_vols\n";
  if ($create_vols) {
    print "Server = $server\n";
    print "partition = $part\n";
  }
}

&load_conf($config_file);

exit(0);

sub 
load_conf {
  local(*FD);
  local($cfile) = @_;
  local($perm, $set_prefix, $set_postfix, $set_beta, $set_sys, $set_delta, $set_dest, $set_files, $set_names, $set_dirs, $set_objprefix);

  open(FD, "$cfile") || 
    die "Can't open configuration file: $cfile: $!\n";
  $perm = 1;
  $set_prefix = 1;
  $set_postfix = 1;
  $set_beta = 1;
  $set_delta = 1;
  $set_dest = 1;
  $set_sys = 1;
  $set_objprefix = 1;

# skip leading foo
  while (<FD>) {
    next if /^#/;
    next if /^\s*$/;

    if (/^\s*names:\s*$/) {
      $set_sys = 0;
      $set_beta = 0;
      $set_postfix = 0;
      $set_objprefix = 0;
      $set_prefix = 0;
      $set_files = 0;
      $set_names = 1;
      $set_dirs = 0;
      next;
    }
    if (/^\s*dirs:\s*$/) {
      $set_sys = 0;
      $set_beta = 0;
      $set_postfix = 0;
      $set_objprefix = 0;
      $set_prefix = 0;
      $set_files = 0;
      $set_names = 0;
      $set_dirs = 1;
      next;
    }
    if (/^\s*files:\s*$/) {
      $set_sys = 0;
      $set_beta = 0;
      $set_postfix = 0;
      $set_objprefix = 0;
      $set_prefix = 0;
      $set_files = 1;
      $set_names = 0;
      $set_dirs = 0;
      next;
    }
    if ($set_names) {
      local($ename, $vname, $remainder);
      ($ename, $vname, $remainder) = split(/\s+/, $_, 3);
      chop $remainder;
      $names{$ename} = $vname;
      $acls{$ename} = $remainder;
    }
    if ($set_dirs) {
      local($dir);
      $dir = eval ("\"$_\"");
      &run("mkdir ${dir}");
    }
    if ($set_files) {
      local($path, $volname, $pathr, $volnamer, $lacl);
      ($path, $volname) = split(/\s+/);
      foreach (sort bylength keys %names) {
        $name = $_;
        $vname = $names{$name};
	$lacl = $acls{$name};
	$pathr = eval ("\"$path\"");
	$volnamer =  eval ("\"$volname\"");
	die "Volume name $name is too long. Volume name cannot exceed 22.\n"
          if (length($volnamer) > 22);
	&mount_vol($pathr, $volnamer, $lacl);
      }
    }

    if ($perm) {
      # first line is the default permissions
      chop;
      $setacl = $_;
      print "Access set to = $setacl\n"
        if ($verbose);
      $perm = 0;
      next;
    }
    if ($set_beta && /^\s*betanameprefix/) {
      $betanameprefix = (split(/\s+/))[1];
      $set_beta = 0;
    }
    if ($set_dest && /^\s*destnameprefix/) {
      $destnameprefix = (split(/\s+/))[1];
      $set_dest = 0;
    }
    if ($set_delta && /^\s*deltanameprefix/) {
      $deltanameprefix = (split(/\s+/))[1];
      $set_delta = 0;
    }
    if ($set_prefix && /^\s*prefix/) {
      $prefix = (split(/\s+/))[1];
      $set_prefix = 0;
    }
    if ($set_objprefix && /^\s*objprefix/) {
      $objprefix = (split(/\s+/))[1];
      $set_objprefix = 0;
    }
    if ($set_postfix && /^\s*postfix/) {
      $postfix = (split(/\s+/))[1];
      $set_postfix = 0;
    }
    if ($set_sys && /^\s*sys/) {
      $sys = (split(/\s+/))[1];
      $set_sys = 0;
    }
  }
  close(FD);
}

sub
bylength {
  split(/\//,$a) <=> split(/\//,$b);
}


sub
mount_vol {
  local($path, $vol, $lacl) = @_;

  if ($create_vols) {
    if (!defined($made{$vol}) || $made{$vol} ne "done") {
      print "Creating $vol...";
      &run("vos create -server $server -partition $part -name $vol");
      print "done.\n";
      $made{$vol} = "done";
    }
  }
  
  print "Mounting $vol at $path...";
  print "\n"
    if ($verbose);
  if (!defined($mounted{$path}) || $mounted{$path} ne "done") {
    &run("fs mkmount -dir $path -vol $vol");
    $mounted{$path} = "done";
  }
  &run("fs checkv");
  &run("fs sq -path $path -max 0");
  &run("fs sa -dir $path -acl $setacl $lacl");
  print "done.\n";
}

sub
run {
  local($cmd) = @_;

  if ($verbose) {
    print "running $cmd\n";
  }
  if ($doit) {
    system($cmd);
    if ($? ne 0) {
      print "Error running: $cmd: rc = $?\n";
      exit(-1);
    }
  }
}

# end of file
