#!/usr/local/bin/perl -w
# $Id: setup-vol.pl,v 1.5 1996/06/20 19:55:44 shadow Exp shadow $
# 
# This perl script reads the configuration file specified on the command line
# and then goes through and (optionally creates), mounts, sets quota and access
# to the specified OS volumes.

$a = '';
$b = '';
$setacl = '';
%files = ();
$doit = 1;
$verbose = 0;
$create_vols = 0;
$server = '';
$part = '';

while ($ARGV[0] =~ /^-/) {
  if ($ARGV[0] eq "-n")	 {
    $doit = 0;
  } elsif ($ARGV[0] eq "-v") {
    $verbose = 1;
  } elsif ($ARGV[0] eq "-c") {
    $create_vols = 1;
    shift @ARGV;
    $server = shift @ARGV;
    $part = @ARGV[0];		# don't shift. will do it at the end of the loop
  } elsif ($ARGV[0] eq "-r") {
    shift @ARGV;
    chdir($ARGV[0]) || die "Unable to chdir to $ARGV[0]: $!\n";
  }
  shift @ARGV;

}

if (@ARGV != 1) {
  print STDERR 
    "usage: setup-vol [-n] [-v] [-c server part] [-r <dir> ] <config_file>\n",
    '$Revision: 1.5 $',
    "\n",
    "\t-n\tDon't really run the commands.\n",
    "\t-v\tBe verbose.\n",
    "\t-c\tCreate the volumes on server/part.\n",
    "\t-r\tSet the root directory to <dir>\n";

  exit(-1);
}

if ($verbose) {
  print "Reading configuration file: @ARGV[0]\n";
  print "Create volumes = $create_vols\n";
  if ($create_vols) {
    print "Server = $server\n";
    print "partition = $part\n";
  }
}

&load_conf(@ARGV[0]);

print "Access set to = $setacl\n"
  if ($verbose);

# process the volumes by length of path
foreach (sort bylength keys %files) {
  &mount_vol($_, $files{$_});
}

exit(0);

sub 
load_conf {
  local(*FD);
  local($cfile) = @_;
  local($files, $perm, $set_prefix, $set_postfix);

  open(FD, "$cfile") || 
    die "Can't open configuration file: $cfile: $!\n";
  $files = 0;
  $perm = 1;
  $set_prefix = 1;
  $set_postfix = 1;

# skip leading foo
  while (<FD>) {
    next if /^#/;
    next if /^\s+$/;

    if ($set_files) {
      local($path, $name);
      ($path, $name) = split(/\s+/);
      $name = "$prefix.$name" 
	if ($prefix);
      $name = "$name.$postfix" 
	if ($postfix);
      die "Volume name $name is too long. Volume name cannot exceed 22.\n"
	if (length($name) > 22);
      $files{$path} = $name;
    }

    if ($perm) {
      # first line is the default permissions
      chop;
      $setacl = $_;
      $perm = 0;
      next;
    }
    if ($set_prefix && /^\s*prefix/) {
      $prefix = (split(/\s+/))[1];
      $set_prefix = 0;
      next;
    }
    if ($set_postfix && /^\s*postfix/) {
      $postfix = (split(/\s+/))[1];
      $set_postfix = 0;
    }
    if (/^\s*files:\s*$/) {
      $set_postfix = 0;
      $set_prefix = 0;
      $set_files = 1;
    }
  }
  close(FD);
}

sub
bylength {
  split(/\//,$a) <=> split(/\//,$b);
}


sub
mount_vol {
  local($path, $vol) = @_;

  if ($create_vols) {
    print "Creating $vol...";
    &run("vos create -server $server -partition $part -name $vol");
    print "done.\n";
  }
  print "Mounting $vol at $path...";
  print "\n"
    if ($verbose);
# evil, but works except on hpux
  &run("mkdir -p $path");
  &run("rmdir $path");
  &run("fs mkmount -dir $path -vol $vol");
  &run("fs sq -path $path -max 0");
  &run("fs sa -dir $path -acl $setacl");
  print "done.\n";
}

sub
run {
  local($cmd) = @_;

  if ($verbose) {
    print "running $cmd\n";
  }
  if ($doit) {
    system($cmd);
    if ($? ne 0) {
      print "Error running: $cmd: rc = $?\n";
      exit(-1);
    }
  }
}

# end of file
