#!/usr/local/bin/perl -w
# $Header: /afs/andrew/system/src/host/pkgtools/002/RCS/soften.pl,v 1.1 1993/12/29 18:23:41 ww0r Exp shadow $

# globals
%files = ();  # file to inode converter
$really = 1;  # really do the action
$verbose = 0; # print out lots o' goop
$path = '.'; # default starting path
$debug = 0;
$preloadfile = "";

&parse_args(@ARGV);
if ($preloadfile ne "") {
  &preload($preloadfile, $path);
}
&traverse($path);

exit 0;

sub
parse_args {
  local(@args) = @_;

  if (@args == 0) {
    $path = ".";
    return;
  }

  while ($args[0] =~ /^-/) {
    if ($args[0] eq "-n") {
      $really = 0;
      $verbose = 1;
    }	
    if ($args[0] eq "-p") {
      shift @args;
      $preloadfile = $args[0];
    }
    if ($args[0] eq "-v") {
      $verbose = 1;
    }
    if ($args[0] eq "-d") {
      $debug = 1;
      $verbose = 1;
    }
    shift @args;
  }
  die "usage: soften [-n] [-v] [-d] [-p preload] [root dir]\n" if ($#args > 0);
  $path = $args[0];
}

# translate $p2 into a relative path to $p1
# this assumes that the path does not contain any '.' or '..' 
# in it.
sub
transpath {
  local($p1, $p2) = @_;
  local(@sp1, @sp2);
  local($count) = 0;
  local($depth);
  local($path);
  
  @sp1 = split(/\/+/, $p1);
  @sp2 = split(/\/+/, $p2);

  while ($sp1[$count] eq $sp2[$count]) {
    $count++;
  }
  print "Common root at $count/$sp1[$count],$sp2[$count]\t" if $debug; 
  $depth = @sp2 - $count - 1; # get rid of filename
  print "gotta go back up $depth steps\n" if $debug;
  $path = '';
  
  for ($i = 0; $i < $depth ; $i++) {
    $path = "../" . $path; 
  }

  $path =  $path . join('/', splice(@sp1, $count));
  $path;
}
  
#
sub 
traverse {
  local($cwd) = @_;
  local($cpath, $path);
  local(*DP);
  local(@stb, $nlinks, $inode);
#  local(@dirs);

  print "Processing $cwd\n";

  next if ($cwd =~ /^\/+dev/);	# ignore /dev directories
  opendir(DP, $cwd) || die "can't open directory $cwd: $!\n";
  foreach (readdir(DP)) {
    next if (/^\.\.?$/);
    $cpath = "$cwd/$_";
    if (-l $cpath ) {
      local($link) = readlink($cpath);
      if ($link eq ".") {
        next;
      }
      # For now. But I'm not sure why we'd ever want to play with a link
      if (-d $link) {
	next;
      }
    }
    if (-d $cpath ) {
#      push(@dirs,$cpath);
      &traverse($cpath);
      next;
    }
    (@stb = lstat($cpath)) || die "can't stat $cpath: $!\n";
    $nlinks = @stb[3];
    $inode = @stb[1];;

    next if ($nlinks == 1);
    # found a hard linked file
    if ($files{$inode}) {
      next if ($files{$inode} eq $cpath);
      print "rm -f $cpath\n" if $verbose;
      if ($really) {
	unlink($cpath) || die "Can't unlink $cpath:$!\n";
      }
      $path = &transpath($files{$inode}, $cpath);
      print "ln -s $path $cpath\n" if $verbose;
      if ($really) {
	symlink($path, $cpath) || die "Can't symlink $path -> $cpath: $!\n";
      }

    } else {
      $files{$inode} = "$cpath";
      print "Found $cpath (inode:$inode/nlinks:$nlinks)\n" if $verbose;
    }
  }
  closedir(DP);
#  foreach (@dirs) {
#    &traverse($_);
#  }
}
  
sub 
preload {
  local($preloadfile, $root) = @_;
  local($cpath, $path);
  local(*DP);
  local(@stb, $nlinks, $inode);

  print "Reading $preloadfile\n";

  open(PRELOAD, "$preloadfile");
  while (<PRELOAD>) {
    chop;
    $cpath = "${root}$_";
    print "Trying ${cpath}\n";
    if (-l $cpath ) {
      local($link) = readlink($cpath);
      if ($link eq ".") {
        next;
      }
      # For now. But I'm not sure why we'd ever want to play with a link
      if (-d $link) {
	next;
      }
    }
    if (-d $cpath ) {
      print ("ignoring directory $cpath\n");
      next;
    }
    (@stb = lstat($cpath)) || die "can't stat $cpath: $!\n";
    $nlinks = @stb[3];
    $inode = @stb[1];;

    next if ($nlinks == 1);
    # found a hard linked file
    if ($files{$inode}) {
      die "Tried to preload multiple links to the same file!";
    } else {
      $files{$inode} = "$cpath";
      print "Found $cpath (inode:$inode/nlinks:$nlinks)\n" if $verbose;
    }
  }
  return;
}
  
